//#include <EEPROM.h>
#include <inttypes.h>
#include <avr/eeprom.h>
#include <avr/io.h>

//digispark
//int pinLED = 1;
//int pinRx  = 0;//Receiver
//int pinRed = 1; //LED
//int pinYel = 2;
//int pinBtn = 3;//Button
//int pinBlu = 4;

//uno
int pinLED = 13;
int pinRed = 3;
int pinYel = 5;
int pinBlu = 6;
int pinRx  = 10;
int pinBtn = 12;

int phase = 0;
int mode = 0;
int btn_val = HIGH;
int btn_state = HIGH;
int btn_cnt = 0;
int btn_cnt_th = 10;
int val_led = 0;
int duration = 0;
int min_duration = 1200;
int max_duration = 1400;

int patern_1_delay = 200;
int patern_1_size = 3;
int patern_1[3][3] =
{
  {1, 0, 0},
  {0, 1, 0},
  {0, 0, 1}
};

int patern_2_delay = 200;
int patern_2_size = 6;
int patern_2[][3] =
{
  {0, 0, 0},
  {1, 0, 0},
  {1, 1, 0},
  {1, 1, 1},
  {0, 1, 1},
  {0, 0, 1}
};

int patern_4_delay = 50;
int patern_4_size = 2;
int patern_4[][3] =
{
  {1, 1, 1},
  {0, 0, 0}
};


void setup() {
  pinMode(pinRx, INPUT);
  pinMode(pinBtn, INPUT_PULLUP); //DIGISPARK NO PULLUP
  pinMode(pinRed, OUTPUT);
  pinMode(pinYel, OUTPUT);
  pinMode(pinBlu, OUTPUT);
  pinMode(pinLED, OUTPUT);
  Serial.begin(9600);
  Serial.println( "start!");

  //EEPROM.get(0, min_duration);
  //EEPROM.get(2, max_duration);
  eeprom_get_int(0, min_duration);
  eeprom_get_int(2, max_duration);


  Serial.print( "min:");
  Serial.println( min_duration);
  Serial.print( "max:");
  Serial.println( max_duration);
  if ( min_duration < 0 ) {
    min_duration = 1100;
        Serial.print( "reset min:");
        Serial.println( min_duration);
  }
  if ( max_duration < 0 ) {
    max_duration = 1100;
        Serial.print( "reset max:");
        Serial.println( max_duration);
  }
}


void loop() {
  btn_val = digitalRead(pinBtn);
  if (btn_val != btn_state) {
    btn_cnt++;
    if (btn_cnt >= btn_cnt_th) {
      btn_cnt = 0;
      btn_state = btn_val;
      if (btn_state == LOW) {
                Serial.println("Calibration. Min/max resetting. ");
        min_duration = 1200;
        max_duration = 1400;
      }
      else {
                Serial.println("Calibration completing. Min/max will be storred to EEPROM. ");
                eeprom_put_int(0, min_duration);
                eeprom_put_int(2, max_duration);
        //        EEPROM.put(0, min_duration);
        //        EEPROM.put(2, max_duration);
      }
    }
  }

  duration = pulseIn(pinRx, HIGH, 10000);

  if (btn_state == 0) {
    if (duration > 0 ) {
      if (duration < min_duration) {
        min_duration = duration;
                Serial.print("min_duration: ");
                Serial.println(min_duration);
      }
      if (duration > max_duration) {
        max_duration = duration;
                Serial.print("max_duration: ");
                Serial.println(max_duration);
      }
    }
    val_led = 1 - val_led;
    digitalWrite(pinLED, val_led ? LOW : HIGH);
    delay(200);
    return;
  }
  else {
    digitalWrite(pinLED, HIGH);
  }

  int val = (duration - min_duration) * 100L / (max_duration - min_duration);

  if (val >= 0 & val < 15 & mode != 0) {
        Serial.println("mode: 0");
    mode = 0;
  }
  if (val >= 16 & val < 32 & mode != 1) {
        Serial.println("mode: 1");
    mode = 1;
  }
  if (val >= 33 & val < 47 & mode != 2) {
    mode = 2;
        Serial.println("mode: 2");
  }
  if (val >= 48 & val < 65 & mode != 3) {
    mode = 3;
        Serial.println("mode: 3");
  }
  if (val >= 66 & val < 83 & mode != 4) {
    mode = 4;
        Serial.println("mode: 4");
  }
  if (val >= 84 & val < 101 & mode != 5) {
    mode = 5                          ;
        Serial.println("mode: 5");
  }
  switch (mode) {
    case 0: //off
      digitalWrite(pinRed, LOW);
      digitalWrite(pinYel, LOW);
      digitalWrite(pinBlu, LOW);
      delay(200);
      break;
    case 1:
      if (phase >= patern_1_size ) phase = 0;
      digitalWrite(pinRed, patern_1[phase][0] ? HIGH : LOW);
      digitalWrite(pinYel, patern_1[phase][1] ? HIGH : LOW);
      digitalWrite(pinBlu, patern_1[phase][2] ? HIGH : LOW);
      delay(patern_1_delay);
      break;
    case 2:
      if (phase >= patern_2_size ) phase = 0;
      digitalWrite(pinRed, patern_2[phase][0] ? HIGH : LOW);
      digitalWrite(pinYel, patern_2[phase][1] ? HIGH : LOW);
      digitalWrite(pinBlu, patern_2[phase][2] ? HIGH : LOW);
      delay(patern_2_delay);
      break;
    case 3: //random
      digitalWrite(pinRed, random(2) ? HIGH : LOW);
      digitalWrite(pinYel, random(2) ? HIGH : LOW);
      digitalWrite(pinBlu, random(2) ? HIGH : LOW);
      delay(random(20, 300));
      break;
    case 4://fast flash
      if (phase >= patern_4_size ) phase = 0;
      digitalWrite(pinRed, patern_4[phase][0] ? HIGH : LOW);
      digitalWrite(pinYel, patern_4[phase][1] ? HIGH : LOW);
      digitalWrite(pinBlu, patern_4[phase][2] ? HIGH : LOW);
      delay(patern_4_delay);
      break;
    case 5: //always on
      digitalWrite(pinRed, HIGH);
      digitalWrite(pinYel, HIGH);
      digitalWrite(pinBlu, HIGH);
      delay(200);
      break;
    default:
      delay(500);

  }
  phase++;
}

void eeprom_get_int( int index, int &val) {
  uint8_t *ptr = (uint8_t*) &val;
  *ptr++ = eeprom_read_byte( (uint8_t*) index++ );
  *ptr = eeprom_read_byte( (uint8_t*) index );
}

void eeprom_put_int(int index, int &val) {
  uint8_t *ptr = (uint8_t*) &val;
  if (eeprom_read_byte( (uint8_t*) index ) != *ptr)
    eeprom_write_byte( (uint8_t*) index, *ptr );
  index++;
  ptr++;
  if (eeprom_read_byte( (uint8_t*) index ) != *ptr)
    eeprom_write_byte( (uint8_t*) index, *ptr );
}

